package javax.swing.plaf.basic;

import javax.swing.text.ViewFactory;
import javax.swing.event.DocumentEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.Position;
import java.awt.Graphics;
import javax.swing.BoundedRangeModel;
import java.awt.Container;
import java.awt.Rectangle;
import javax.swing.JTextField;
import java.awt.Shape;
import javax.swing.text.ParagraphView;
import java.awt.Component;
import java.awt.Insets;
import javax.swing.text.JTextComponent;
import javax.swing.text.FieldView;
import javax.swing.text.GlyphView;
import javax.swing.text.View;
import javax.swing.text.Element;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;

public class BasicTextFieldUI extends BasicTextUI
{
    public static ComponentUI createUI(JComponent component) {
        return new BasicTextFieldUI();
    }
    
    public BasicTextFieldUI() {
    }
    
    @Override
    protected String getPropertyPrefix() {
        return "TextField";
    }
    
    @Override
    public View create(final Element elem) {
        if (Boolean.TRUE.equals(elem.getDocument().getProperty("i18n"))) {
            String name = elem.getName();
            if (name != null) {
                if (name.equals("content")) {
                    return new GlyphView(elem) {
                        @Override
                        public float getMinimumSpan(int axis) {
                            return this.getPreferredSpan(axis);
                        }
                    };
                }
                if (name.equals("paragraph")) {
                    return new I18nFieldView(elem);
                }
            }
        }
        return new FieldView(elem);
    }
    
    @Override
    public int getBaseline(JComponent c, int width, int height) {
        super.getBaseline(c, width, height);
        View rootView = this.getRootView((JTextComponent)c);
        if (rootView.getViewCount() > 0) {
            Insets insets = c.getInsets();
            height = height - insets.top - insets.bottom;
            if (height > 0) {
                int top = insets.top;
                View view = rootView.getView(0);
                int n = (int)view.getPreferredSpan(1);
                if (height != n) {
                    top += (height - n) / 2;
                }
                int n2;
                if ((view instanceof BasicTextFieldUI.I18nFieldView)) {
                    int baseline = BasicHTML.getBaseline(view, width - insets.left - insets.right, height);
                    if (baseline < 0) {
                        return -1;
                    }
                    n2 = top + baseline;
                }
                else {
                    n2 = top + c.getFontMetrics(c.getFont()).getAscent();
                }
                return n2;
            }
        }
        return -1;
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior(JComponent c) {
        super.getBaselineResizeBehavior(c);
        return Component.BaselineResizeBehavior.CENTER_OFFSET;
    }
    
    static class I18nFieldView extends ParagraphView
    {
        I18nFieldView(Element elem) {
            super(elem);
        }
        
        @Override
        public int getFlowSpan(int n) {
            return Integer.MAX_VALUE;
        }
        
        @Override
        protected void setJustification(int n) {
        }
        
        static boolean isLeftToRight(Component component) {
            return component.getComponentOrientation().isLeftToRight();
        }
        
        Shape adjustAllocation(Shape shape) {
            if (shape != null) {
                Rectangle bounds = shape.getBounds();
                int n = (int)this.getPreferredSpan(1);
                int n2 = (int)this.getPreferredSpan(0);
                if (bounds.height != n) {
                    int n3 = bounds.height - n;
                    bounds.y += n3 / 2;
                    bounds.height -= n3;
                }
                Container container = this.getContainer();
                if ((container instanceof JTextField)) {
                    BoundedRangeModel horizontalVisibility = ((JTextField)container).getHorizontalVisibility();
                    int max = Math.max(n2, bounds.width);
                    int value = horizontalVisibility.getValue();
                    int min = Math.min(max, bounds.width - 1);
                    if (value + min > max) {
                        value = max - min;
                    }
                    horizontalVisibility.setRangeProperties(value, min, horizontalVisibility.getMinimum(), max, false);
                    if (n2 < bounds.width) {
                        int n4 = bounds.width - 1 - n2;
                        int horizontalAlignment = ((JTextField)container).getHorizontalAlignment();
                        if (isLeftToRight(container)) {
                            if (horizontalAlignment == 10) {
                                horizontalAlignment = 2;
                            }
                            else if (horizontalAlignment == 11) {
                                horizontalAlignment = 4;
                            }
                        }
                        else if (horizontalAlignment == 10) {
                            horizontalAlignment = 4;
                        }
                        else if (horizontalAlignment == 11) {
                            horizontalAlignment = 2;
                        }
                        switch (horizontalAlignment) {
                            case 0: {
                                bounds.x += n4 / 2;
                                bounds.width -= n4;
                                break;
                            }
                            case 4: {
                                bounds.x += n4;
                                bounds.width -= n4;
                                break;
                            }
                        }
                    }
                    else {
                        bounds.width = n2;
                        bounds.x -= horizontalVisibility.getValue();
                    }
                }
                return bounds;
            }
            else {
                return null;
            }
        }
        
        void updateVisibilityModel() {
            Container container = this.getContainer();
            if ((container instanceof JTextField)) {
                BoundedRangeModel horizontalVisibility = ((JTextField)container).getHorizontalVisibility();
                int a = (int)this.getPreferredSpan(0);
                int extent = horizontalVisibility.getExtent();
                int max = Math.max(a, extent);
                int n = (extent == 0) ? max : extent;
                int a2 = max - n;
                int value = horizontalVisibility.getValue();
                if (value + n > max) {
                    value = max - n;
                }
                horizontalVisibility.setRangeProperties(Math.max(0, Math.min(a2, value)), n, 0, max, false);
            }
        }
        
        @Override
        public void paint(Graphics g, Shape shape) {
            Rectangle rectangle = (Rectangle)shape;
            g.clipRect(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
            super.paint(g, this.adjustAllocation(shape));
        }
        
        @Override
        public int getResizeWeight(int n) {
            if (n == 0) {
                return 1;
            }
            return 0;
        }
        
        @Override
        public Shape modelToView(int pos, Shape shape, Position.Bias b) throws BadLocationException {
            return super.modelToView(pos, this.adjustAllocation(shape), b);
        }
        
        @Override
        public Shape modelToView(int p5, Position.Bias b0, int p6, Position.Bias b2, Shape shape) throws BadLocationException {
            return super.modelToView(p5, b0, p6, b2, this.adjustAllocation(shape));
        }
        
        @Override
        public int viewToModel(float x, float y, Shape shape, Position.Bias[] bias) {
            return super.viewToModel(x, y, this.adjustAllocation(shape), bias);
        }
        
        @Override
        public void insertUpdate(DocumentEvent changes, Shape shape, ViewFactory f) {
            super.insertUpdate(changes, this.adjustAllocation(shape), f);
            this.updateVisibilityModel();
        }
        
        @Override
        public void removeUpdate(DocumentEvent changes, Shape shape, ViewFactory f) {
            super.removeUpdate(changes, this.adjustAllocation(shape), f);
            this.updateVisibilityModel();
        }
    }
}
